use flair::flair;
use ssw::websrv::WebsrvConfig;
use ssw::websrv::run_server;
use std::fs;
use std::env;
use std::collections::HashMap;
use std::{thread, time};

//1 - Ingest all the pages into a variable
// : all the pages must start with a number indicating their order
//2 - build the index with the pages
//3 - Put all the generated content inside the html folder
//4 - Start the server and serve

//#1 : Sends (pages_summary, pages_content)
fn ingest_pages(pages_path: &str, 
                header_path: &str, 
                footer_path: &str,
                html_path: &str,
                ) -> Option<(String, String)> {

    //Getting the name of existing pages
    let mut pages: Vec<String> = Vec::new();
    for file in fs::read_dir(pages_path).unwrap() {
        //println!("{:#?}", file.unwrap().path());
        let mypage = String::from(file
                                  .unwrap()
                                  .path()
                                  .file_name()
                                  .unwrap()
                                  .to_str()
                                  .unwrap()
            );

        let _ = &mut pages.push(mypage);
    }
    pages.sort();
    pages.reverse();

    if pages.len() > 0 {
        //Handling content
        let mut pages_content = String::new();
        let mut page_sum = String::new();
        for page in pages {
            let vars: HashMap<String, String> = HashMap::from([
                (String::from("header"), fs::read_to_string(header_path).unwrap()),
                (String::from("footer"), fs::read_to_string(footer_path).unwrap()),
            ]);
            let fullpath = String::from(format!("{pages_path}/{page}"));
            let page_res = flair::analyze_file(&fullpath, vars).unwrap();
            page_sum.push_str(&page_link(&page, &page_res, html_path));
            let _ = &mut pages_content.push_str(&page_res);
        }
        return Some((page_sum, pages_content));
    } else {
        return None;
    }
}

fn page_link(page_name: &str, content: &String, html_path: &str) -> String {
    let page_html = str::replace(page_name, ".ft", "");
    let page_loc = format!("{html_path}/pages/{page_html}");
    let page_name_short = str::replace(page_name, ".html.ft", "");
    let page_link = format!("<a href=\"pages/{page_html}\">{page_name_short}</a><br>\n");

    match fs::write(page_loc, content) {
        Ok(_) => (),
        Err(e) => panic!("{}", e),
    };
    return String::from(page_link);
}


//#2 & #3
fn generate_index(pages: (String, String),
                  index_template_path: &str,
                  html_path: &str,
                  header_path: &str,
                  footer_path: &str,
                  ) {

    let index_path = format!("{html_path}/index.html");

    let vars: HashMap<String, String> = HashMap::from([
        (String::from("header"), fs::read_to_string(&header_path).unwrap()),
        (String::from("footer"), fs::read_to_string(&footer_path).unwrap()),
        (String::from("pages"), pages.1),
        (String::from("sum"), pages.0),
    ]);

    let index = flair::analyze_file(index_template_path, vars).unwrap();
    
    fs::write(&index_path, index.as_str()).expect("Could not write index");
}

fn generate_content() -> String {
    //Initialize from env variables
    let root_path_env = "FLOG_ROOT_FOLDER";
    let root_folder = match env::var(root_path_env) {
        Ok(v) => v,
        Err(_) => String::from("/opt/flog"),
    };

    let pages_path = format!("{root_folder}/templates/pages");
    let html_folder_path = format!("{root_folder}/html");
    let index_template_path = format!("{root_folder}/templates/index.html.ft");
    let header_path = format!("{html_folder_path}/header.html");
    let footer_path = format!("{html_folder_path}/footer.html");

    let pages = match ingest_pages(&pages_path, &header_path, &footer_path, &html_folder_path) {
        Some(p) => {
            println!("INFO : Generated site's content");
            p
        },
        None => {
            println!("ERR : No pages found, nothing to ingest");
            (String::new(), String::new())
        },
    };

    generate_index(pages, &index_template_path, &html_folder_path, &header_path, &footer_path);
    return html_folder_path;
}


fn main() {
    let html_folder = generate_content();
    //Web conf
    let webconf = WebsrvConfig {
        nbr_of_threads: 10,
        bind_addr: String::from("0.0.0.0:8000"),
        root_folder: String::from(&html_folder),
    };
    run_server(webconf);
    loop {
        let time_wait = time::Duration::from_secs(600);
        thread::sleep(time_wait);
        generate_content();
        continue;
    }


}
